/** 									// -o-
 * Circle.pde
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


// ------------------------------------------------ -o--
class Circle extends Button  
{
  String  className = "Circle"; 

  int  centerX, centerY, radius; 	// dimentions: center X/Y, radius
  int  radiSq;


  // ------------------------- -o-
  // constructors
  //
  Circle() { /*EMPTY*/ } 

  Circle(String path_, String color_, int x_, int y_, int radius_)
  {
    super(path_, color_);
    setCirc(x_, y_, radius_);
  }

  private void setCirc(int x_, int y_, int radius_) 			// -o-
  {
    centerX = x_; 		
    centerY = y_;
    radius  = radius_;

    radiSq = radius*radius; 	// (speedup)

    checkBounds();

    pixelDiffCountMinimum = 2*radius / _pixelDiffCountRatio;
    	// ASSUME  circle is always symmetric across both x- and y-axes
  }


  protected void checkBounds() 						// -o-
  {
    if (! (((centerX-radius) >=0) && ((centerX+radius) < width)
	&& ((centerY-radius) >=0) && ((centerY+radius) < height)) )
    {
      this.ERRORLN( className + "() is out of bounds -- "
		      +"x="+ centerX +" y=" + centerY +" radius=" + radius );
    }
  }  // checkBounds()



  // ------------------------- -o-
  // helpers
  //
  String toString()  { return toString(""); }

  String toString(String prefix) 
  {
    String  class_desc = className + "(" + osc_path +", "+ colorName +")";
    String  class_info = "xy=("+ centerX +","+ centerY +") r="+ radius;

    if ( (prefix != null) && (0 < prefix.length()) ) 
    {
      class_desc = prefix +" = "+ class_desc;
    } 

    return super.toString(class_desc, class_info);

  }  // toString()


  Circle display()							
  {
    super.display();
    ellipseMode(RADIUS);
    ellipse(centerX, centerY, radius, radius);

    if (true == text_enabled) { 
      fill(colorVal, COLOR_SOLID);
      text(text_string, centerX, centerY +textHOffset); 
    }

    return this;
  }  // display()



  // ------------------------- -o-
  // methods
  //
  boolean isActive() 							// -o-
  {
    if (isDelayed())  { return true; }

    boolean differenceConfirmed = false;
    int diffcount = 0;

    scan_difflist:
    for (int y = centerY-radius; y <= centerY+radius; y++) {
      for (int x = centerX-radius; x <= centerX+radius; x++) {
	if ( (pow(centerX-x,2) + (pow(centerY-y,2))) <= radiSq )
	{
	  if (1 == difflist[(y*width)+x]) 
	  {
	    if (pixelDiffCountMinimum < ++diffcount) {
	      differenceConfirmed = true;
	      break scan_difflist;
	    }
	  }
        }  // endif -- within circle radius
      }  // endfor -- x
    }  // endfor -- y


    if (differenceConfirmed) 
    { 	
      trigger_count += 1;
      if (osc_trigger) { oscSend(); }
      setDelayTime(0); 	// positive hit automatically resets delayInterval
    }

    return differenceConfirmed;

  }  // isActive()

}  // class Circle

