/** 									// -o-
 * overlay_tablecloth.pde
 *
 *   Cheap, simple, almost musical.
 *
 *
 *   void setupOsc()
 *
 *   void defineTriggers()
 *   void setupTriggers()
 *   void displayTriggers()
 *   void dumpTriggers()
 *   void scanTriggers()
 *
 * To establish a trigger:
 *   1  declare 
 *   2  define in defineTriggers(); 
 *        add dynamic system values in setupTriggers() 
 *   3  list in displayTriggers() and dumpTriggers()
 *   4  list in scanTriggers(); add state logic here.
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


/*
  TBD --
    . change colors during delay
    . harmonize?
    . ...
*/


// -------------------------------------------- -o--
// setupOsc()
//
// establish port for receiving OSC messages; establish IP address
// and port for sending OSC messages.
//
// CALLED BY: setup()
//
void setupOsc()
{
  // receiving port
  //
  osc = new OscP5(this, 42002); 

  // target IP and port
  //
  osc_target = new NetAddress("127.0.0.1", 57120); 	// sc canonical
}



// -------------------------------------------- -o--
// declare and define triggers
//

Circle[][] car; 	

int objcnt_w = 0, 
    objcnt_h = 0;


int enable_grid = 1;



// CALLED BY: setup()
//
void defineTriggers() 							// -o--
{
  // -o-
  // grid of circles 
  //
   
  int  i, j;

  int  radius = 25,
       trigger_spacer = 13,
       array_max = 50;

  float  rand_base = random(50,250),
         rand_low = rand_base + random(50, 150),
         rand_high = rand_base + random(3000,8000),
         rand_delay = random(10000, 25000);

  Circle tmp;

  car = new Circle[array_max][array_max];


  for (	i = radius+trigger_spacer, objcnt_w = 0; 
  	i < (cam.width-radius); 
	i += ((radius*2)+trigger_spacer), objcnt_w++ )
  {
    for (	j = radius+trigger_spacer, objcnt_h = 0;
    		j < (cam.height-radius); 
		j += ((radius*2)+trigger_spacer), objcnt_h++ )
    {
      tmp = car[objcnt_w][objcnt_h] = 
      			new Circle("/thing1", "greygreen", i, j, radius);

      tmp.oscMsg( round100(random(rand_low, rand_high)) );
      tmp.delay((int) rand_delay);

    }  // endfor j
  }  // endfor i


  NOTELN( "objcnt_wh: "+ objcnt_w +"*"+ objcnt_h +" == "+ objcnt_w*objcnt_h);
  NOTELN( "RAND:  base=" + round100(rand_base)
   	    +" low="+ round100(rand_low)
	    +" high="+ round100(rand_high) 
	    +"  ("+ round100(rand_high-rand_low) +")" 
	);
  NOTELN( "DELAY: "+ (int) rand_delay );

}  // defineTriggers()




// -------------------------------------------- -o--
// setup, display and dump triggers
//

// CALLED BY: setup()
//
void setupTriggers() 
{
  // define list of colors
  //
  colorlist = new HashMap();

  colorlist.put( "blue", 	#0000ff );
  colorlist.put( "greygreen", 	#004411 );
  colorlist.put( "green", 	#00ff00 );
  colorlist.put( "turquoise", 	#00ffff );
  colorlist.put( "forestgreen", #0f5f3a );
  colorlist.put( "brightlime", 	#0fff55 );
  colorlist.put( "darkblue", 	#111133 );
  colorlist.put( "brown", 	#222200 );
  colorlist.put( "limegreen", 	#4e9a1e );
  colorlist.put( "purple", 	#8400b4 );
  colorlist.put( "red", 	#ff0000 );

  colorlist.put( "newcolor", 	#1080ff );

}  // setupTriggers()



// XXX  list all triggers intended to appear in the display
//
// CALLED BY: draw()
//
void displayTriggers() 							// -o--
{
  if (1 == enable_grid) 
  {
    for (int i = 0; i < objcnt_w; i += 1) {
      for (int j = 0; j < objcnt_h; j += 1) {
        car[i][j].display();
      }
    }
  }

}  // displayTriggers()



// XXX  list all triggers that should be tracked
//
void dumpTriggers() 							// -o--
{
  println( "\nTRIGGERS--");
  println( "  (runtime: " + timestamp_ms(-1) +")\n" );

  if (1 == enable_grid) {
    for (int i = 0; i < objcnt_w; i += 1) {
      for (int j = 0; j < objcnt_h; j += 1) {
	println(car[i][j].toString("car-"+i+"-"+j) +"\n");
      }
    }
  }

}  // dumpTriggers()




// -------------------------------------------- -o--
// scan triggers and handle algebra of triggers
//
// CALLED BY: draw()
//

int activeCount = 0,
    activeMod = 11000,
    activeModLast = activeMod;


void scanTriggers()
{
  // XXX  list all triggers intended to be activated by pixel differences;
  // NB  isActive() calls isDelayed() which maintains delayMode state
  //

  if (1 == enable_grid) 
  {
    for (int i = 0; i < objcnt_w; i += 1) {
      for (int j = 0; j < objcnt_h; j += 1) {
	if (true == car[i][j].isActive()) {
	  activeCount += 1; 	// XXX  active triggers counted multiple times
	  //NOTELN("" + activeCount);
	}
      }
    }

    // XXX  skips most mod intervals
    //      q: is this function being interrupted?
    //      q: how to guarantee order of execution?
    //
    if (activeCount >= activeModLast) {
      println();
      NOTELN("ACTIVE: count="+ activeCount +" modLast="+ activeModLast);

      activeModLast += activeMod;
      defineTriggers();
    }
  }

}  // scanTriggers()

